@@ -23,10 +23,9 @@
 #include <xorg-server.h>
 #include <xorgVersion.h>
 
+#include <wacom-util.h>
 #include "Xwacom.h"
 
-#define ARRAY_SIZE(a) (sizeof(a)/sizeof((a)[0]))
-
 /* max number of input events to read in one read call */
 #define MAX_EVENTS 50
 
@@ -38,6 +37,12 @@
 #include <xf86Xinput.h>
 #include <mipointer.h>
 #include <X11/Xatom.h>
+/*****************************************************************************
+ * Unit test hack
+ ****************************************************************************/
+#ifdef DISABLE_STATIC
+#define static
+#endif
 
 /******************************************************************************
  * Debugging support
@@ -51,8 +56,8 @@
 #define DBG(lvl, priv, ...) \
 	do { \
 		if ((lvl) <= priv->debugLevel) { \
-			xf86Msg(X_INFO, "%s (%s): ", \
-				((WacomDeviceRec*)priv)->name, __func__); \
+			xf86Msg(X_INFO, "%s (%d:%s): ", \
+				((WacomDeviceRec*)priv)->name, lvl, __func__); \
 			xf86Msg(X_NONE, __VA_ARGS__); \
 		} \
 	} while (0)
@@ -60,21 +65,6 @@
 #define DBG(lvl, priv, ...)
 #endif
 
-/*****************************************************************************
- * General Macros
- ****************************************************************************/
-
-#define ABS(x) ((x) > 0 ? (x) : -(x))
-
-/*****************************************************************************
- * General Defines
- ****************************************************************************/
-#define XI_STYLUS "STYLUS"      /* X device name for the stylus */
-#define XI_CURSOR "CURSOR"      /* X device name for the cursor */
-#define XI_ERASER "ERASER"      /* X device name for the eraser */
-#define XI_PAD    "PAD"         /* X device name for the Pad */
-#define XI_TOUCH  "TOUCH"       /* X device name for the touch */
-
 /******************************************************************************
  * WacomModule - all globals are packed in a single structure to keep the
  *               global namespaces as clean as possible.
@@ -86,10 +76,10 @@ struct _WacomModule
 	InputDriverPtr wcmDrv;
 
 	int (*DevOpen)(DeviceIntPtr pWcm);
-	void (*DevReadInput)(LocalDevicePtr local);
+	void (*DevReadInput)(InputInfoPtr pInfo);
 	void (*DevControlProc)(DeviceIntPtr device, PtrCtrl* ctrl);
-	int (*DevChangeControl)(LocalDevicePtr local, xDeviceCtl* control);
-	void (*DevClose)(LocalDevicePtr local);
+	int (*DevChangeControl)(InputInfoPtr pInfo, xDeviceCtl* control);
+	void (*DevClose)(InputInfoPtr pInfo);
 	int (*DevProc)(DeviceIntPtr pWcm, int what);
 	int (*DevSwitchMode)(ClientPtr client, DeviceIntPtr dev, int mode);
 };
@@ -114,76 +104,87 @@ struct _WacomModule
 #define SYSCALL(call) while(((call) == -1) && (errno == EINTR))
 
 /* Open the device with the right serial parmeters */
-extern Bool wcmOpen(LocalDevicePtr local);
+extern Bool wcmOpen(InputInfoPtr pInfo);
 
 /* device autoprobing */
-char *wcmEventAutoDevProbe (LocalDevicePtr local);
+char *wcmEventAutoDevProbe (InputInfoPtr pInfo);
 
 /* common tablet initialization regime */
-int wcmInitTablet(LocalDevicePtr local, const char* id, float version);
+int wcmInitTablet(InputInfoPtr pInfo, const char* id, float version);
 
 /* standard packet handler */
-void wcmReadPacket(LocalDevicePtr local);
+void wcmReadPacket(InputInfoPtr pInfo);
 
 /* handles suppression, filtering, and dispatch. */
 void wcmEvent(WacomCommonPtr common, unsigned int channel, const WacomDeviceState* ds);
 
 /* dispatches data to XInput event system */
-void wcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds);
+void wcmSendEvents(InputInfoPtr pInfo, const WacomDeviceState* ds);
 
 /* generic area check for xf86Wacom.c, wcmCommon.c and wcmXCommand.c */
 Bool wcmPointInArea(WacomToolAreaPtr area, int x, int y);
 Bool wcmAreaListOverlap(WacomToolAreaPtr area, WacomToolAreaPtr list);
 
 /* calculate the proper tablet to screen mapping factor */
-void wcmMappingFactor(LocalDevicePtr local);
+void wcmMappingFactor(InputInfoPtr pInfo);
 
 /* validation */
-extern Bool wcmIsAValidType(LocalDevicePtr local, const char* type);
+extern Bool wcmIsAValidType(InputInfoPtr pInfo, const char* type);
 extern Bool wcmIsWacomDevice (char* fname);
-extern int wcmIsDuplicate(char* device, LocalDevicePtr local);
-extern int wcmDeviceTypeKeys(LocalDevicePtr local);
+extern int wcmIsDuplicate(char* device, InputInfoPtr pInfo);
+extern int wcmDeviceTypeKeys(InputInfoPtr pInfo);
 
 /* hotplug */
-extern int wcmNeedAutoHotplug(LocalDevicePtr local, const char **type);
-extern void wcmHotplugOthers(LocalDevicePtr local, const char *basename);
+extern int wcmNeedAutoHotplug(InputInfoPtr pInfo, const char **type);
+extern void wcmHotplugOthers(InputInfoPtr pInfo, const char *basename);
 
 /* setup */
-extern int wcmParseOptions(LocalDevicePtr local, int hotplugged);
-extern void wcmInitialCoordinates(LocalDevicePtr local, int axes);
-extern void wcmInitialScreens(LocalDevicePtr local);
-extern void wcmInitialScreens(LocalDevicePtr local);
+extern Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary, Bool is_dependent);
+extern Bool wcmPostInitParseOptions(InputInfoPtr pInfo, Bool is_primary, Bool is_dependent);
+extern int wcmParseSerials(InputInfoPtr pinfo);
+extern void wcmInitialCoordinates(InputInfoPtr pInfo, int axes);
+extern void wcmInitialScreens(InputInfoPtr pInfo);
+extern void wcmInitialScreens(InputInfoPtr pInfo);
 
-extern int wcmDevSwitchModeCall(LocalDevicePtr local, int mode);
+extern int wcmDevSwitchModeCall(InputInfoPtr pInfo, int mode);
 extern int wcmDevSwitchMode(ClientPtr client, DeviceIntPtr dev, int mode);
 
 /* run-time modifications */
-extern void wcmChangeScreen(LocalDevicePtr local, int value);
-extern void wcmTilt2R(WacomDeviceStatePtr ds);
-extern void wcmGestureFilter(WacomDevicePtr priv, int channel);
+extern void wcmChangeScreen(InputInfoPtr pInfo, int value);
+extern int wcmTilt2R(int x, int y, double offset);
 extern void wcmEmitKeycode(DeviceIntPtr keydev, int keycode, int state);
-extern void wcmSoftOutEvent(LocalDevicePtr local);
+extern void wcmSoftOutEvent(InputInfoPtr pInfo);
 
-extern void wcmRotateTablet(LocalDevicePtr local, int value);
-extern void wcmRotateCoordinates(LocalDevicePtr local, int* x, int* y);
-extern void wcmVirtualTabletSize(LocalDevicePtr local);
-extern void wcmVirtualTabletPadding(LocalDevicePtr local);
+extern void wcmRotateTablet(InputInfoPtr pInfo, int value);
+extern void wcmRotateAndScaleCoordinates(InputInfoPtr pInfo, int* x, int* y);
+extern void wcmVirtualTabletSize(InputInfoPtr pInfo);
+extern void wcmVirtualTabletPadding(InputInfoPtr pInfo);
 
 extern int wcmCheckPressureCurveValues(int x0, int y0, int x1, int y1);
 extern int wcmGetPhyDeviceID(WacomDevicePtr priv);
 
 /* device properties */
 extern int wcmSetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop, BOOL checkonly);
-extern void InitWcmDeviceProperties(LocalDevicePtr local);
+extern int wcmGetProperty(DeviceIntPtr dev, Atom property);
+extern int wcmDeleteProperty(DeviceIntPtr dev, Atom property);
+extern void InitWcmDeviceProperties(InputInfoPtr pInfo);
+extern void wcmUpdateRotationProperty(WacomDevicePtr priv);
+extern void wcmUpdateSerial(InputInfoPtr pInfo, unsigned int serial);
 
 /* Utility functions */
-extern Bool is_absolute(LocalDevicePtr local);
-extern void set_absolute(LocalDevicePtr local, Bool absolute);
+extern Bool is_absolute(InputInfoPtr pInfo);
+extern void set_absolute(InputInfoPtr pInfo, Bool absolute);
 extern WacomCommonPtr wcmRefCommon(WacomCommonPtr common);
 extern void wcmFreeCommon(WacomCommonPtr *common);
 extern WacomCommonPtr wcmNewCommon(void);
 
+enum WacomSuppressMode {
+	SUPPRESS_NONE = 8,	/* Process event normally */
+	SUPPRESS_ALL,		/* Supress and discard the whole event */
+	SUPPRESS_NON_MOTION	/* Supress all events but x/y motion */
+};
+
 /****************************************************************************/
 #endif /* __XF86WACOM_H */
 
-/* vim: set noexpandtab shiftwidth=8: */
+/* vim: set noexpandtab tabstop=8 shiftwidth=8: */
